* VERSION HISTORY
****************************

****************************
****************************
* CFSR 3: RISK-STANDARDIZED PERFORMANCE 
* - RECURRENCE OF MALTREATMENT
****************************
****************************

* This syntax is provided for informational purposes only. It requires access to 
* child-level data from all states, the District of Columbia, and Puerto Rico.  

* The following syntax is used to calculate an individual state's performance 
* for a recent cohort of children against the national standard and the 
* historical cohorts (from all other states) that were used to establish the 
* national standard. Although this syntax will calculate RSPs and data for all 
* states, the only results of interest are for the state being evaluated 
* (MyState "XX"). 

****************************
* SPECIFY THE STATE AND 12-MONTH COHORT WHOSE PERFORMANCE IS BEING ASSESSED
****************************

* Note: For the 12-month cohort specified (CurrentCohort), there must exist a 
* file for that same cohort in "C:\cfsr3\Performance observed child". This file 
* will contain observed performance for a recent cohort of children (from all 
* states). For example, if the user specifies CurrentCohort = 1314, their must 
* be a file called "CFSR 3 - Observed perf for maltx recurrence 1314.dta."

set more off
*set trace on
*set min_memory 16g

****************************
* USER INPUT REQUIRED
****************************

* Specify root folder where your files and folders are. 
* This folder must have these subfolders:
* ... \Performance modeled
* ... \Performance observed child
* ... \Fixed files

local dirstub "C:\CFSR 3\Analysis - DP 2019 July"

* Specify 12-month cohort
local CurrentCohortList "1718"

* Specify state(s) to run. 
* Option 1 is to list them below in the local line:
 local statelist PA NY  
 local statetxtlist Pennsylvania "New York" 
 
* Option 2 is to list them in a dataset (which can be conveniently used for the other indicators).
*use "`dirstub'/1 - Fixed files/States 2017.dta", clear
*levelsof stateabb, local(statelist) 
*levelsof statetxt, local(statetxtlist)
*local statelist
*local statetxtlist
*forvalues i = 1/`=_N' {
*     local statelist `"`statelist' `"`=stateabb[`i']'"'"'
*	 local statetxtlist `"`statetxtlist' `"`=statetxt[`i']'"'"'
* }
 
****************************
* RUN SYNTAX FROM HERE TO END
****************************

foreach CurrentCohort of local CurrentCohortList {

local numelements : word count `statelist'
forvalue i = 1/`numelements'{
local MyState : word `i' of `statelist'
local MyStateTxt : word `i' of `statetxtlist'

* create log file. 
log using "`dirstub'/4 - Performance modeled/RSP for maltx recurrence `MyState' `CurrentCohort'.txt", text replace
 
****************************
* GET SOURCE FILE and PREP FILE
****************************

* Open the current cohort file. This file contains observed performance for a 
* recent cohort of children (from all states). 
use "`dirstub'/3 - Performance observed child/CFSR 3 - Observed perf for maltx recurrence `CurrentCohort'.dta", clear
keep if stateabb == `"`MyState'"' 

* Flag if state requested is missing from the current cohort data file (usually due to failing DQ).
quietly count
local DQState = 0
if r(N) == 0 {
	local DQState = 1
}

tempfile holding
save `holding'


* Open the historical cohort file. This file contains the fixed national 
* standard for the indicator (Perf_Nation) and the observed performance for the 
* 12-month historical cohort of children that was used to establish the national 
* standard. We will be assessing the state's performance with its most recent 
* cohort (currently stored in 'holding') against the national standard and 
* historical cohorts from all other states. This syntax assumes the historical
* cohort file is saved in the Fixes files folder of the dirstub path defined earlier.
macro list
use "`dirstub'/1 - Fixed files/CFSR 3 - NS Observed perf for maltx recurrence 1213.dta", clear

quietly summarize ChildAge, detail
local ChildAge_Med = r(p50)
local Perf_NationNS = Perf_Nation[1]

* IF State requested does not have a current cohort score, creat a special DQ 
* output for that state rather than run the rest of the code.
if `DQState' == 1 {
	clear
	set obs 10	
	generate stateabb = `"`MyState'"'
	generate state = `"`MyStateTxt'"'
	generate Indicator_new = "Recurrence of maltreatment"
	generate NS = "" + string(`Perf_NationNS'*100,"%02.1f")+"%"
	generate TwelveMoCohort = "FY" + "20" + substr(`"`CurrentCohort'"',1,2)
	generate Period = "FY" + substr(`"`CurrentCohort'"',1,2) + "-" + substr(`"`CurrentCohort'"',3,2)
	generate DataUsed = Period
	generate Format = "Percent"
	generate RSP_NS = ""
	generate RSP_Range = ""
	generate RSP_Range_Formatted = ""
	generate RSP_Pip = ""
	generate ValueType = "DataUsed2" in 1
	replace ValueType = "Den_State" in 2
	replace ValueType = "Num_State" in 3
	replace ValueType = "Perf_NationRnd" in 4
	replace ValueType = "Perf_StateRnd" in 5
	replace ValueType = "RSP_NS2" in 6
	replace ValueType = "RspLowRnd" in 7
	replace ValueType = "RSP_Range2" in 8
	replace ValueType = "RSPRnd" in 9
	replace ValueType = "RSPUppRnd" in 10
	generate Result = 0
	replace Result = round(`Perf_NationNS',0.001) in 4
	generate Result_String = "DQ"
	replace Result_String = "FY" + substr(`"`CurrentCohort'"',1,2) + " - " + substr(`"`CurrentCohort'"',3,2) in 1
	replace Result_String = "" in 4
	replace Result_String = "" in 6
	generate MatchField = ""
	*Save file
	save "`dirstub'/4 - Performance modeled/CFSR 3 - RSP for maltx recurrence - `CurrentCohort' `MyState'.dta", replace
	outsheet stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
	using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MalRecur_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace
	export excel stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
	using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MalRecur_Summary `CurrentCohort' `MyState'.xlsx", nolabel replace firstrow (variables)
	*close log and return to the top of the loop
	 log close
	 continue
}

drop if stateabb == `"`MyState'"'
append using `holding'

* For the state being evaluated, replace the value it has for national observed
* performance (which is based on the current cohort for all states) with the 
* fixed NS for this indicator. The level of precision matters for the 
* calculations RSP, RSPUpp, and RSPLow matters.
replace Perf_Nation = `Perf_NationNS' if stateabb == `"`MyState'"'

sort state

* Describe dataset
* Pick one observation per state
* Count and verify number of states in file
* Calculate desired direction for performance on this indicator
* Verify desired 12-month cohort has been selected

describe
egen pickone = tag(stateabb)
count if pickone
gen Passing = "Below NS"
tab Perf_Nation
tab TwelveMoCohort

****************************
* PREDICT OUTCOME
****************************

* Run multi-level logit model predicting maltreatment recurrence (Num_Child = 1)
* Adjust for child age at initial victimization (ChildAge)
* The median age is used as the reference group. 

timer on 1
xtmelogit Num_Child ib(`ChildAge_Med').ChildAge, baselevels || state:, variance
timer off 1
timer list 1
timer clear

predict xb, xb 
predict re, reffects 
predict rese, reses 

* xb = child's predicted log odds of recurrence based on child's age, but 
*    without considering the child's home state (i.e., ignoring the state's 
*    random effect)
* re = state's random effect (shift in child's predicted log odds of 
*    recurrence after considering the child's home state; aka, Empirical 
*    Bayes intercept)
* rese - standard error of state's random effect 

****************************
* CALCULATE RISK-STANDARDIZED PERFORMANCE
****************************

* 1. Calculate PREDICTED number of recurrences in each state
****************************

* The predicted number of recurrences based on the state's performance with 
* its observed case mix. This is our best prediction of future performance, 
* assuming no change in case mix or policy. It is calculated as the sum of each 
* child's predicated probability of recurrence, which is a complex function 
* of the child's value of xb and his or her state's specific random effect. 
 
sort state
gen double Child_Pred = exp(xb+re)/(1+exp(xb+re)) 
by state: egen double Num_Pred = total(Child_Pred) 

* 2. Calculate EXPECTED number of recurrences in each state
****************************

* The expected number of recurrences based on the nation's performance with 
* the state's case mix. This represents how many recurrences we would expect
* for the state's children if they were treated in an "average" state. It is 
* calculated as the sum of each child's predicted probability of recurrence
* (xb), including the *average* intercept of all states (the average of the 
* random effects across states is zero). 

gen double Child_Exp = exp(xb)/(1+exp(xb))
by state: egen double Num_Exp = total(Child_Exp)

* 3. Calculate risk-standardized ratio and risk-standardized performance
****************************

* Calculate ratio of predicted to expected
* Multiply ratio by national observed performance (i.e., the national standard)
* to get RSP
gen double Ratio_PE = Num_Pred / Num_Exp
gen double RSP = (Num_Pred / Num_Exp) * Perf_Nation

* 4. Calculate 95% confidence intervals for RSP
****************************

* Upper CI
sort state
gen double UppNum = exp(xb+re+(1.96*rese))/(1+exp(xb+re+(1.96*rese)))
by state: egen double UppNumSum = total(UppNum)
gen double UppDen = exp(xb)/(1+exp(xb))
by state: egen double UppDenSum = total(UppDen)
gen double RSPUpp = (UppNumSum/UppDenSum) * Perf_Nation

* Lower CI
gen double LowNum = exp(xb+re-(1.96*rese))/(1+exp(xb+re-(1.96*rese)))
by state: egen double LowNumSum = total(LowNum)
gen double LowDen = exp(xb)/(1+exp(xb))
by state: egen double LowDenSum = total(LowDen)
gen double RSPLow = (LowNumSum/LowDenSum) * Perf_Nation

****************************
* COMPARE STATE'S RSP TO NATIONAL STANDARD (NS)
****************************

* When comparing state's RSP (actually, the CIs) to the national observed
* performance, use rounded versions.

* Round the national observed performance 
clonevar Perf_NationRnd = Perf_Nation
replace Perf_NationRnd = round(Perf_NationRnd,0.001)

* Round the CIs of the RSP
clonevar RSPLowRnd = RSPLow
clonevar RSPUppRnd = RSPUpp
replace RSPLowRnd = round(RSPLowRnd,0.001)
replace RSPUppRnd = round(RSPUppRnd,0.001)

* Compare CI's of the RSP relative to national observed performance
gen RSP_NS = "No diff"
replace RSP_NS = "Met" if RSPUppRnd < Perf_NationRnd
replace RSP_NS = "Not met" if RSPLowRnd > Perf_NationRnd

* Count number of states meeting, not meeting, and no different from NS
egen RSP_Met=total(RSP_NS=="Met" & pickone)
egen RSP_NotMet=total(RSP_NS=="Not met" & pickone)
egen RSP_NotDif=total(RSP_NS=="No diff" & pickone)

* Count number of states that must engage in a PIP
gen RSP_Pip = ""
replace RSP_Pip = "No PIP" if RSP_NS == "Met"
replace RSP_Pip = "No PIP" if RSP_NS == "No diff"
replace RSP_Pip = "PIP" if RSP_NS == "Not met"
 

* Create variable holding median age at initial victimization to include in 
* output. The coded age values are one higher than the actual age (e.g., a coded 
* value of 7 represents an actual age of 6), so median age = coded value for 
* median age minus 1.  
sort state
by state: egen MedAge = median(ChildAge)
replace MedAge = MedAge -1

* Save
save "`dirstub'/4 - Performance modeled/CFSR 3 - RSP for maltx recurrence - `CurrentCohort' `MyState'.dta", replace

****************************
* REPORTING 
****************************

gen Indicator_new = "Recurrence of maltreatment"

* Round the RSP and Perf_State to 1 decimal (so it's, e.g., .421; avoids having 
* to format it later in Excel to show only 1 decimal)
clonevar RSPRnd = RSP
replace RSPRnd = round(RSPRnd,0.001)
clonevar Perf_StateRnd = Perf_State
replace Perf_StateRnd = round(Perf_StateRnd,0.001)

* gen RSP_new = round(RSP,0.001)*100 (e.g., "40.1")
gen RSP_Range = string(RSPLowRnd*100,"%02.1f") + "-" + string(RSPUppRnd*100,"%02.1f")
gen RSP_Range_Formatted = string(RSPLowRnd*100,"%02.1f") + "%-" + string(RSPUppRnd*100,"%02.1f") + "%"
gen Format = "Percent"
gen Period = "FY" + substr(`"`CurrentCohort'"',1,2) + "-" + substr(`"`CurrentCohort'"',3,2)
gen DataUsed = "FY" + substr(`"`CurrentCohort'"',1,2) + "-" + substr(`"`CurrentCohort'"',3,2)

* NOTE These variables only exist to ensure that for tableau there is both a column and record with this data after reshaping below.
gen NS = string(`Perf_NationNS'*100,"%02.1f")+"%"
gen RSP_Range2 = RSP_Range_Formatted
gen RSP_NS2 = RSP_NS
gen DataUsed2 = DataUsed
gen MatchField = ""

outsheet stateabb statetxt Indicator_new NS TwelveMoCohort Period DataUsed DataUsed2 Format RSP_NS RSP_NS2 RSP_Range RSP_Range_Formatted RSP_Range2 RSP_Pip Den_State Num_State Perf_StateRnd RSPLowRnd RSPRnd RSPUppRnd Perf_NationRnd MatchField ///
if (stateabb == `"`MyState'"' & pickone==1) using "`dirstub'/4 - Performance modeled/RSP_MalRecur_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace


import delimited "`dirstub'/4 - Performance modeled/RSP_MalRecur_Summary `CurrentCohort' `MyState'.csv", clear case(preserve)

rename (Den_State Num_State Perf_StateRnd RSPLowRnd RSPRnd RSPUppRnd Perf_NationRnd) Result=
rename (RSP_Range2 RSP_NS2 DataUsed2) str=

reshape long Result str, i(stateabb statetxt) j(ValueType) string

replace Result = 0 if missing(Result)
 

rename (str) Result_String
rename (statetxt) state

outsheet stateabb	state	Indicator_new	NS	TwelveMoCohort	Period	DataUsed	Format	RSP_NS	RSP_Range	RSP_Range_Formatted	RSP_Pip	ValueType	Result	Result_String MatchField ///
 using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MalRecur_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace
 
export excel stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
 using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MalRecur_Summary `CurrentCohort' `MyState'.xlsx", nolabel replace firstrow (variables)
	
*close log and return to the top of the loop
 log close
}
}
