* VERSION HISTORY
****************************

****************************
****************************
* CFSR 3: RISK-STANDARDIZED PERFORMANCE 
* - RE-ENTRY TO FOSTER CARE IN 12 MONTHS
****************************
****************************

* This syntax is provided for informational purposes only. It requires access to 
* child-level data from all states, the District of Columbia, and Puerto Rico.  

* The following syntax is used to calculate an individual state's performance 
* for a recent cohort of children against the national standard and the 
* historical cohorts (from all other states) that were used to establish the 
* national standard. Although this syntax will calculate RSPs and data for all 
* states, the only results of interest are for the state being evaluated 
* (MyState "XX").   

****************************
* SPECIFY THE STATE AND 12-MONTH COHORT WHOSE PERFORMANCE IS BEING ASSESSED
****************************

* Note: For the 12-month cohort specified (CurrentCohort), there must exist a 
* file for that same cohort in "C:\cfsr3\Performance observed child". This file 
* will contain observed performance for a recent cohort of children (from all 
* states). For example, if the user specifies CurrentCohort = AB12, their must 
* be a file called "CFSR 3 - Observed perf for reentry AB12.dta."

set more off
set trace off
*set min_memory 16g

****************************
* USER INPUT REQUIRED
****************************

* Specify root folder where your files and folders are. 
* This folder must have these subfolders:
* ... \Performance modeled
* ... \Performance observed child
* ... \Fixed files

local dirstub "C:/CFSR 3/Analysis - DP 2018 March"

* Specify 12-month cohort
local CurrentCohortList  AB15 BA16
*
* Specify state(s) to run. 
* Option 1 is to list them below in the local line:
 local statelist MI MS
 local statetxtlist Michigan Mississippi

* Option 2 is to list them in a dataset (which can be conveniently used for the 
* other indicators).
*use "`dirstub'/1 - Fixed files/States 2017.dta", clear
*local statelist
*local statetxtlist
*forvalues i = 1/`=_N' {
     *local statelist `"`statelist' `"`=stateabb[`i']'"'"'
	 *local statetxtlist `"`statetxtlist' `"`=statetxt[`i']'"'"'
 *}

****************************
* RUN SYNTAX FROM HERE TO END
****************************
macro list
foreach CurrentCohort of local CurrentCohortList {

local numelements : word count `statelist'

*forvalue i = 1/1{
forvalue i = 1/`numelements'{
local MyState : word `i' of `statelist'
local MyStateTxt : word `i' of `statetxtlist'

* create log file. 
log using "`dirstub'/4 - Performance modeled/RSP for reentry `MyState' `CurrentCohort'.txt", text replace

****************************
* GET SOURCE FILE and PREP FILE
****************************

* Open the current cohort file. This file contains observed performance for a 
* recent cohort of children (from all states). 
use "`dirstub'/3 - Performance observed child/CFSR 3 - Observed perf for reentry `CurrentCohort'.dta", clear
keep if stateabb == `"`MyState'"' 

* Flag if state requested is missing from the current cohort data file (usually due to failing DQ).
quietly count
local DQState = 0
if r(N) == 0 {
	local DQState = 1
}

tempfile holding
save `holding'

* Open the historical cohort file. This file contains the fixed national 
* standard for the indicator (Perf_Nation) and the observed performance for the 
* 12-month historical cohort of children that was used to establish the national 
* standard. We will be assessing the state's performance with its most recent 
* cohort (currently stored in 'holding') against the national standard and 
* historical cohorts from all other states. This syntax assumes the historical
* cohort file is saved in the Fixes files folder of the dirstub path defined earlier.

use "`dirstub'/1 - Fixed files/CFSR 3 - NS Observed perf for reentry BA12.dta", clear


quietly summarize ChildAge, detail
local ChildAge_Med = r(p50)
local Perf_NationNS = Perf_Nation[1]


* IF State requested does not have a current cohort score, creat a special DQ 
* output for that state rather than run the rest of the code.
if `DQState' == 1 {
	clear
	set obs 10	
	generate stateabb = `"`MyState'"'
	generate state = `"`MyStateTxt'"'
	generate Indicator_new = "Re-entry to foster care"
	generate NS = "" + string(`Perf_NationNS'*100,"%02.1f")+"%"
	generate TwelveMoCohort = `"`CurrentCohort'"'
	if substr(`"`CurrentCohort'"',1,2) ==  "AB"{
               gen Period = substr(`"`CurrentCohort'"',3,2) + "A" + substr(`"`CurrentCohort'"',3,2) + "B"
			   gen DataUsed = substr(`"`CurrentCohort'"',3,2) + "A-" + string(real(substr(`"`CurrentCohort'"',3,2))+2) + "B"
}
else if substr(`"`CurrentCohort'"',1,2) == "BA" {
               gen Period = string(real(substr(`"`CurrentCohort'"',3,2))-1) + "B" + substr(`"`CurrentCohort'"',3,2) + "A"
			   gen DataUsed = string(real(substr(`"`CurrentCohort'"',3,2))-1) + "B-" + string(real(substr(`"`CurrentCohort'"',3,2))+2) + "A"
}
	generate Format = "Percent"
	generate RSP_NS = ""
	generate RSP_Range = ""
	generate RSP_Range_Formatted = ""
	generate RSP_Pip = ""
	generate ValueType = "DataUsed2" in 1
	replace ValueType = "Den_State" in 2
	replace ValueType = "Num_State" in 3
	replace ValueType = "Perf_NationRnd" in 4
	replace ValueType = "Perf_StateRnd" in 5
	replace ValueType = "RSP_NS2" in 6
	replace ValueType = "RspLowRnd" in 7
	replace ValueType = "RSP_Range2" in 8
	replace ValueType = "RSPRnd" in 9
	replace ValueType = "RSPUppRnd" in 10
	generate Result = 0
	replace Result = round(`Perf_NationNS',0.001) in 4
	generate Result_String = "DQ"
	replace Result_String = DataUsed in 1
	replace Result_String = "" in 4
	replace Result_String = "" in 6
	generate MatchField = ""
	*Save file
	save "`dirstub'/4 - Performance modeled/CFSR 3 - RSP for reentry - `CurrentCohort' `MyState'.dta", replace
	outsheet stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
	using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_Reentry_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace
	export excel stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
	using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_Reentry_Summary `CurrentCohort' `MyState'.xlsx", nolabel replace firstrow (variables)
	*close log and return to the top of the loop
	 log close
	 continue
}

drop if stateabb == `"`MyState'"'
append using `holding'

* For the state being evaluated, replace the value it has for national observed
* performance (which is based on the current cohort for all states - .083279099181) with the 
* fixed NS for this indicator. The level of precision matters for the 
* calculations RSP, RSPUpp, and RSPLow matters.

replace Perf_Nation = `Perf_NationNS' if stateabb == `"`MyState'"'

sort state

* Describe dataset
* Pick one observation per state
* Count and verify number of states in file
* Calculate desired direction for performance on this indicator
* Verify desired 12-month cohort has been selected

describe
egen pickone = tag(stateabb)
count if pickone
gen Passing = "Below NS"
tab Perf_Nation
tab TwelveMoCohort

****************************
* PREDICT OUTCOME
****************************

* Run multi-level logit model predicting re-entry by 12 months (NumRE_Child = 1)
* Adjust for child age at exit (ChildAge) and state's entry rate (EntryRate)
* The median age is used as the reference group. 

xtmelogit NumRE_Child ib(`ChildAge_Med').ChildAge EntryRate, baselevels || state:, variance

predict xb, xb 
predict re, reffects 
predict rese, reses 

* xb = child's predicted log odds of re-entry based on child's age and the 
*    state-specific entry rate, but without considering the child's home state
*    (i.e., ignoring the state's random effect)
* re = state's random effect (shift in child's predicted log odds of 
*    re-entry after considering the child's home state; aka, Empirical 
*    Bayes intercept)
* rese - standard error of state's random effect

****************************
* CALCULATE RISK-STANDARDIZED PERFORMANCE
****************************

* 1. Calculate PREDICTED number of outcomes
****************************

* The predicted number of re-entries based on state's performance with its 
* observed case mix. This is our best prediction of future performance, 
* assuming no change in case mix or policy. It is calculated as the sum of each 
* child's predicated probability of re-entry, which is a complex function of the child's value of 
* xb and his or her state's specific random effect. 
 
sort state
gen double Child_Pred = exp(xb+re)/(1+exp(xb+re)) 
by state: egen double Num_Pred = total(Child_Pred) 

* 2. Calculate EXPECTED number of outcomes
****************************

* The expected number of re-entries based on the nation's performance with 
* the state's case mix. This represents how many re-entries we would expect
* for the state's children if they were treated in an "average" state. It is 
* calculated as the sum of each child's predicted probability of re-entry 
* (xb), including the *average* intercept of all states (the average of the 
* random effects across states is zero).

gen double Child_Exp = exp(xb)/(1+exp(xb))
by state: egen double Num_Exp = total(Child_Exp)

* 3. Calculate risk-standardized ratio and risk-standardized performance
****************************

* Calculate ratio of predicted to expected
* Multiply ratio by national observed performance (i.e., the national standard)
* to get RSP
gen double Ratio_PE = Num_Pred / Num_Exp
gen double RSP = (Num_Pred / Num_Exp) * Perf_Nation

* 4. Calculate 95% confidence intervals for RSP
****************************

* Upper CI
sort state
gen double UppNum = exp(xb+re+(1.96*rese))/(1+exp(xb+re+(1.96*rese)))
by state: egen double UppNumSum = total(UppNum)
gen double UppDen = exp(xb)/(1+exp(xb))
by state: egen double UppDenSum = total(UppDen)
gen RSPUpp = (UppNumSum/UppDenSum) * Perf_Nation

* Lower CI
gen double LowNum = exp(xb+re-(1.96*rese))/(1+exp(xb+re-(1.96*rese)))
by state: egen double LowNumSum = total(LowNum)
gen double LowDen = exp(xb)/(1+exp(xb))
by state: egen double LowDenSum = total(LowDen)
gen double RSPLow = (LowNumSum/LowDenSum) * Perf_Nation

****************************
* COMPARE STATE'S RSP TO NATIONAL STANDARD
****************************

* When comparing the state's RSP (actually, the CIs) to the national observed
* performance, use rounded versions.

* Round the national observed performance 
clonevar Perf_NationRnd = Perf_Nation
replace Perf_NationRnd = round(Perf_NationRnd,0.001)

* Round the CIs of the RSP
clonevar RSPLowRnd = RSPLow
clonevar RSPUppRnd = RSPUpp
replace RSPLowRnd = round(RSPLowRnd,0.001)
replace RSPUppRnd = round(RSPUppRnd,0.001)

* Compare CI's of the RSP relative to national observed performance
gen RSP_NS = "No diff"
replace RSP_NS = "Met" if RSPUppRnd < Perf_NationRnd
replace RSP_NS = "Not met" if RSPLowRnd > Perf_NationRnd

* Count number of states meeting, not meeting, and no different from NS
egen RSP_Met=total(RSP_NS=="Met" & pickone)
egen RSP_NotMet=total(RSP_NS=="Not met" & pickone)
egen RSP_NotDif=total(RSP_NS=="No diff" & pickone)

* Count number of states that must engage in a PIP
gen RSP_Pip = ""
replace RSP_Pip = "No PIP" if RSP_NS == "Met"
replace RSP_Pip = "No PIP" if RSP_NS == "No diff"
replace RSP_Pip = "PIP" if RSP_NS == "Not met"

****************************
* REPORTING  
****************************

* Export to excel the state's results, formatted for profile
gen Indicator_new = "Re-entry to foster care"
gen RSPRnd = round(RSP,0.001) 
gen Perf_StateRnd = round(Perf_State,0.001)

gen RSP_Range = string(RSPLowRnd*100,"%02.1f") + "-" + string(RSPUppRnd*100,"%02.1f")
gen RSP_Range_Formatted = string(RSPLowRnd*100,"%02.1f") + "%-" + string(RSPUppRnd*100,"%02.1f") + "%"

gen Format = "Percent"
if substr(`"`CurrentCohort'"',1,2) ==  "AB"{
               gen Period = substr(`"`CurrentCohort'"',3,2) + "A" + substr(`"`CurrentCohort'"',3,2) + "B"
			   gen DataUsed = substr(`"`CurrentCohort'"',3,2) + "A-" + string(real(substr(`"`CurrentCohort'"',3,2))+2) + "B"
}
else if substr(`"`CurrentCohort'"',1,2) == "BA" {
               gen Period = string(real(substr(`"`CurrentCohort'"',3,2))-1) + "B" + substr(`"`CurrentCohort'"',3,2) + "A"
			   gen DataUsed = string(real(substr(`"`CurrentCohort'"',3,2))-1) + "B-" + string(real(substr(`"`CurrentCohort'"',3,2))+2) + "A"
}

* These variables only exist to ensure that there is both a column and record with this data after reshaping below, and because tableau.
gen NS = string(`Perf_NationNS'*100,"%02.1f")+"%"
gen RSP_Range2 = RSP_Range_Formatted
gen RSP_NS2 = RSP_NS
gen DataUsed2 = DataUsed
gen MatchField = ""

outsheet stateabb statetxt Indicator_new NS TwelveMoCohort Period DataUsed DataUsed2 Format RSP_NS RSP_NS2 RSP_Range RSP_Range_Formatted RSP_Range2 RSP_Pip Den_State Num_State Perf_StateRnd RSPLowRnd RSPRnd RSPUppRnd Perf_NationRnd MatchField ///
if (stateabb == `"`MyState'"' & pickone==1) using "`dirstub'/4 - Performance modeled/RSP_Reentry_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace


import delimited "`dirstub'/4 - Performance modeled/RSP_Reentry_Summary `CurrentCohort' `MyState'.csv", clear case(preserve)

rename (Den_State Num_State Perf_StateRnd RSPLowRnd RSPRnd RSPUppRnd Perf_NationRnd) Result=
rename (RSP_Range2 RSP_NS2 DataUsed2) str=

reshape long Result str, i(stateabb statetxt) j(ValueType) string
replace Result = 0 if missing(Result)
 
rename (str) Result_String
rename (statetxt) state

outsheet stateabb	state	Indicator_new	NS	TwelveMoCohort	Period	DataUsed	Format	RSP_NS	RSP_Range	RSP_Range_Formatted	RSP_Pip	ValueType	Result	Result_String MatchField ///
 using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_Reentry_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace
 
export excel stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
 using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_Reentry_Summary `CurrentCohort' `MyState'.xlsx", nolabel replace firstrow (variables)


log close
}
}
